# -*- coding: utf-8 -*-
# (c) 2021 The mqttwarn developers
from dataclasses import dataclass
from typing import Dict, List, Union


@dataclass
class ProcessorItem:
    """
    A processor item for feeding information into service handlers.
    """

    service: str = None
    target: str = None
    config: Dict = None
    addrs: List[str] = None
    priority: int = None
    topic: str = None
    title: str = None
    message: Union[str, bytes] = None
    data: Dict = None



import logging

import pytest

from mqttwarn.core import Service


@pytest.fixture
def mqttwarn_service():
    """
    A service instance for propagating to the plugin.
    """
    logger = logging.getLogger(__name__)
    return Service(mqttc=None, logger=logger)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'


import pynsca
from pynsca import NSCANotifier


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    config   = item.config

    statii = [ pynsca.OK, pynsca.WARNING, pynsca.CRITICAL, pynsca.UNKNOWN ]
    status = pynsca.OK
    try:
        prio = item.priority
        status = statii[prio]
    except:
        pass

    nsca_host = str(config['nsca_host'])

    host_name = item.addrs[0]
    service_description = item.addrs[1]

    # If the incoming payload has been transformed, use that,
    # else the original payload
    text = item.message

    try:
        notif = NSCANotifier(nsca_host)
        notif.svc_result(host_name, service_description, status, text)
    except Exception as e:
        srv.logging.warning("Cannot notify to NSCA host `%s': %s" % (nsca_host, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import nntplib
from six import StringIO
from email.mime.text import MIMEText
from email.utils import formatdate


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    host      = item.config.get('server', 'localhost')
    port      = item.config.get('port', 119)
    username  = item.config.get('username')
    password  = item.config.get('password')

    try:
        from_hdr = item.addrs[0]
        newsgroup = item.addrs[1]
    except Exception:
        srv.logging.error("Incorrect target configuration for %s" % item.target)
        return False

    try:

        text  = item.message
        title    = item.title

        msg = MIMEText(text)

        msg['From']         = from_hdr
        msg['Subject']      = item.title
        msg['Newsgroups']   = newsgroup
        msg['Date']         = formatdate()
        msg['User-Agent']   = srv.SCRIPTNAME
        # msg['Message-ID'] = '<jp001@tiggr>'

        msg_file = StringIO(msg.as_string())
        nntp = nntplib.NNTP(host, port, user=username, password=password)

        srv.logging.debug(nntp.getwelcome())
        nntp.set_debuglevel(0)

        nntp.post(msg_file)
        nntp.quit()
    except Exception as e:
        srv.logging.warn("Cannot post to %s newsgroup: %s" % (newsgroup, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Philipp Adelt <autosort-github@philipp.adelt.net>, based on code by Jan Badenhorst'
__copyright__ = 'Copyright 2016 Philipp Adelt, 2014 Jan Badenhorst'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import os

try:
    import simplejson as json
except ImportError:
    import json

import gspread
import oauth2client.client
import oauth2client.file
from oauth2client import clientsecrets


SCOPE = "https://spreadsheets.google.com/feeds"


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    try:
        spreadsheet_url = item.addrs[0]
        worksheet_name = item.addrs[1]
        client_secrets_filename = item.config['client_secrets_filename']
        oauth2_code = item.config['oauth2_code']
        oauth2_storage_filename = item.config['oauth2_storage_filename']
    except KeyError as e:
        srv.logging.error("Some configuration item is missing: %s" % e)
        return False

    if not os.path.exists(client_secrets_filename):
        srv.logging.error("Cannot find file '%s'." % client_secrets_filename)
        return False

    try:
        srv.logging.debug("Adding row to spreadsheet %s [%s]..." % (spreadsheet_url, worksheet_name))
        if os.path.isfile(oauth2_storage_filename):
            # Valid credentials from previously completed authentication?
            srv.logging.debug("Trying to use credentials from file '%s'." %
                oauth2_storage_filename)
            storage = oauth2client.file.Storage(oauth2_storage_filename)
            credentials = storage.get()
            if credentials is None or credentials.invalid:
                srv.logging.error("Error reading credentials from file '%s'." %
                    oauth2_storage_filename)
                return False
        elif oauth2_code is not None and len(oauth2_code) > 0:
            # After restart - hopefully with the code coming from the Google webpage.
            srv.logging.debug("Trying to use client_secrets from '%s' and OAuth code '%s'." %
                (client_secrets_filename, oauth2_code))
            try:
                credentials = oauth2client.client.credentials_from_clientsecrets_and_code(
                    client_secrets_filename,
                    scope=SCOPE,
                    code=oauth2_code,
                    redirect_uri='urn:ietf:wg:oauth:2.0:oob')
                if credentials is None:
                    raise clientsecrets.InvalidClientSecretsError("Resulting credentials are None!?")
            except clientsecrets.InvalidClientSecretsError as e:
                srv.logging.error("Something went wrong using '%s' and OAuth code '%s': %s" %
                    (client_secrets_filename, oauth2_code, e))
                return False
            except oauth2client.client.FlowExchangeError as e:
                if 'invalid_grantCode' in e.message:
                    srv.logging.error("It seems you need to start over: Clear the "
                        "'oauth2_code'-field and restart mqttwarn.")
                    return False
                else:
                    raise e

            # Store credentials for next event.
            storage = oauth2client.file.Storage(oauth2_storage_filename)
            storage.put(credentials)

        else:
            # Start a new authentication flow and scream the URL to visit to the logs.
            flow = oauth2client.client.flow_from_clientsecrets(
                client_secrets_filename,
                scope=SCOPE,
                redirect_uri='urn:ietf:wg:oauth:2.0:oob')
            auth_uri = flow.step1_get_authorize_url()
            srv.logging.error('NO AUTHENTICATION AVAILABLE: Visit this URL and copy code to '
                'mqttwarn.ini -> config:gss2 -> oauth2_code: %s' % auth_uri)
            return False

        gc = gspread.authorize(credentials)
        wks = gc.open_by_url(spreadsheet_url).worksheet(worksheet_name)
        col_names = wks.row_values(1)

        # Column names found need to be keys in item.data to end up in the new row.
        values = []
        for col in col_names:
            values.append(item.data.get(col, ""))

        wks.append_row(values)

        srv.logging.debug("Successfully added row to spreadsheet")

    except Exception as e:
        srv.logging.warn("Error adding row to spreadsheet %s [%s]: %s" % (spreadsheet_url, worksheet_name, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import errno
import subprocess


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    # addrs is a list[] with process name and args
    argv = item.addrs

    text = item.message

    if not text.endswith("\n"):
        text = text + "\n"

    try:
        proc = subprocess.Popen(argv,
            stdin=subprocess.PIPE, close_fds=True)
    except Exception as e:
        srv.logging.warn("Cannot create pipe: %s" % e)
        return False

    try:
        proc.stdin.write(text.encode('utf-8'))
    except IOError as e:
        srv.logging.warn("Cannot write to pipe: errno %d" % (e.errno))
        return False
    except Exception as e:
        srv.logging.warn("Cannot write to pipe: %s" % e)
        return False

    proc.stdin.close()
    proc.wait()
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from twilio.rest import TwilioRestClient


def plugin(srv, item):
    """ expects (accountSID, authToken, from, to) in addrs"""

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    try:
        account_sid, auth_token, from_nr, to_nr = item.addrs
    except:
        srv.logging.warn("Twilio target is incorrectly configured")
        return False

    text = item.message

    try:
        client = TwilioRestClient(account_sid, auth_token)
        message = client.messages.create(
                    body=text,
                    to=to_nr,
                    from_=from_nr)
        srv.logging.debug("Twilio returns %s" % (message.sid))
    except Exception as e:
        srv.logging.warn("Twilio failed: %s" % e)
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Orhan Hirsch <orhanhenrik()gmail.com>'
__copyright__ = 'Copyright 2017 Orhan Hirsch'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import json
import celery


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    config = item.config

    app = celery.Celery(
        config['app_name'],
        broker=config['broker_url']
    )

    for target in item.addrs:
        message = item.message
        try:
            if target['message_format'] == 'json':
                message = json.loads(message)
            app.send_task(target['task'], [message])
        except Exception as e:
            srv.logging.warning("Error: %s" % e)
            return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

try:
    import simplejson as json
except ImportError:
    import json
from mqttwarn.vendor import ZabbixSender
import time


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    try:
        trapper, port = item.addrs
    except:
        srv.logging.error("Module target is incorrectly configured")
        return False

    host = item.config.get('host', 'MQTT_BUS')
    discovery_key = item.config.get('discovery_key', 'mqtt.discovery')

    client = item.data.get('client', None)
    if client is None:
        srv.logging.warn("No client in item; ignoring")
        return False

    # If status_key is in item (set by function ZabbixData()), then we have to
    # add a host to Zabbix LLD, and the value of the message is 0/1 to indicate
    # host down/up
    status_key = item.data.get('status_key', None)
    if status_key is not None:
        status = item.message

        clients = []
        clients.append( { '{#MQTTHOST}' : client } )

        # {"data": [{"{#MQTTHOST}": "jog02"}]}
        lld_payload = json.dumps(dict(data = clients))

        try:
            # Add LLD for the client host to Zabbix
            sender = ZabbixSender.ZabbixSender(trapper, server_port = int(port))
            sender.AddData(host=host, key=discovery_key, value=lld_payload)
            res = sender.Send()
            sender.ClearData()

            if res and 'info' in res:
                srv.logging.debug("Trapper for LLD responds with %s" % res['info'])

                # Add status to the "status key". This must not happen too early,
                # or Zabbix will fail this value if the LLD for the host hasn't
                # been recorded. Attempt a sleep. FIXME
                time.sleep(3)  # FIXME
                sender.AddData(host=client, key=status_key, value=status)
                res = sender.Send()
                sender.ClearData()
                if res and 'info' in res:
                    srv.logging.debug("Trapper for STATUS responds with %s" % res['info'])

                return True
        except Exception as e:
            srv.logging.warn("Trapper responded: %s" % e)
            return False

    # We are adding a normal item/value via the trapper
    key = item.data.get('key', None)
    if client is None or key is None:
        srv.logging.warn("Client or Key missing in item; ignoring")
        return False

    value = item.message

    try:
        # Send item/value for host to the trapper
        sender = ZabbixSender.ZabbixSender(trapper, server_port = int(port))
        sender.AddData(host=client, key=key, value=value)
        res = sender.Send()
        sender.ClearData()

        if res and 'info' in res:
            srv.logging.debug("Trapper for client=%s, item=%s, value=%s responds with %s" % (client, key, value, res['info']))

            return True
    except Exception as e:
        srv.logging.warn("Trapper responded: %s" % e)

    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import str

__author__ = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2015 Jan-Piet Mens'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import dns.update
import dns.query
import dns.tsigkeyring


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    config = item.config

    dns_nameserver = config['dns_nameserver']
    dns_keyname = config['dns_keyname']
    dns_keyblob = config['dns_keyblob']

    try:
        zone, domain, ttl, rrtype = item.addrs

    except:
        srv.logging.error("Incorrect target configuration for {0}/{1}".format(item.service, item.target))
        return False

    text = item.message

    try:
        keyring = dns.tsigkeyring.from_text({str(dns_keyname): str(dns_keyblob)});

        update = dns.update.Update(zone,
                                   keyring=keyring,
                                   keyname=dns_keyname,
                                   keyalgorithm='hmac-sha256')  # FIXME configurable

        if rrtype.upper() == 'TXT':
            text = '"%s"' % text

        update.replace(domain, ttl, rrtype, text)
        response = dns.query.tcp(update, dns_nameserver, timeout=10)

        srv.logging.debug("DNS Update: {0}".format(dns.rcode.to_text(response.rcode())))
    except Exception as e:
        srv.logging.warning("Cannot notify to dnsupdate `%s': %s" % (dns_nameserver, e))
        return False

    return True
